// Copyright 2022 Bitcoding
// See the License for the specific language governing permissions and
// limitations under the License.

package com.btcpiyush.ads.google_applovin_unity_ads;

import android.util.Log;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.applovin.mediation.MaxAd;
import com.applovin.mediation.MaxAdListener;
import com.applovin.mediation.MaxError;
import com.applovin.mediation.ads.MaxAppOpenAd;
import com.applovin.sdk.AppLovinSdk;
import com.google.android.gms.ads.AdError;
import com.google.android.gms.ads.LoadAdError;

import java.lang.ref.WeakReference;

import io.flutter.util.Preconditions;

/**
 * A wrapper for {@link MaxAppOpenAd}.
 */
class FlutterAppLovinOpenAd extends FlutterAd.FlutterOverlayAd {

    private static final String TAG = "FlutterAppOpenAd";

    @NonNull
    private final AdInstanceManager manager;
    @NonNull
    private final String adUnitId;
    @NonNull
    private final AppLovinSdk _appLovinSDK;
    @Nullable
    private MaxAppOpenAd ad;
    @NonNull
    private final FlutterAdLoader flutterAdLoader;

    FlutterAppLovinOpenAd(
            int adId,
            @NonNull AdInstanceManager manager,
            @NonNull String adUnitId,
            @NonNull AppLovinSdk _appLovinSDK,
            @NonNull FlutterAdLoader flutterAdLoader) {
        super(adId);
        this.manager = manager;
        this.adUnitId = adUnitId;
        this._appLovinSDK = _appLovinSDK;
        this.flutterAdLoader = flutterAdLoader;
    }

    @Override
    void load() {
        ad = flutterAdLoader.loadAppLovinOpen(
                adUnitId,
                _appLovinSDK,
                new DelegatingAppOpenAdLoadCallback(this));
    }


    private void onAdLoaded() {
        manager.onAdLoaded(adId, null);
    }

    private void onAdFailedToLoad(@NonNull LoadAdError loadAdError) {
        manager.onAdFailedToLoad(adId, new FlutterLoadAdError(loadAdError));
    }

    @Override
    void show() {
        if (ad == null) {
            Log.w(TAG, "Tried to show app open ad before it was loaded");
            return;
        }
        if (manager.getActivity() == null) {
            Log.e(TAG, "Tried to show app open ad before activity was bound to the plugin.");
            return;
        }

        if (ad.isReady()) {
            ad.showAd();
        } else {
            manager.onFailedToShowFullScreenContent(adId, new AdError(999, "AppOpen is not ready", ""));
        }
    }

    @Override
    void setImmersiveMode(boolean immersiveModeEnabled) {

    }

    @Override
    void dispose() {
        if (ad != null) {
            ad.destroy();
        }
        ad = null;
    }

    /**
     * An AppOpenAdLoadCallback that just forwards events to a delegate.
     */
    private static final class DelegatingAppOpenAdLoadCallback implements MaxAdListener {

        private final WeakReference<FlutterAppLovinOpenAd> delegate;

        DelegatingAppOpenAdLoadCallback(FlutterAppLovinOpenAd delegate) {
            this.delegate = new WeakReference<>(delegate);
        }

        @Override
        public void onAdLoaded(MaxAd ad) {
            if (delegate.get() != null) {
                delegate.get().onAdLoaded();
            }
        }

        @Override
        public void onAdDisplayed(MaxAd ad) {
            if (delegate.get() != null) {
                delegate.get().manager.onAdShowedFullScreenContent(delegate.get().adId);
            }
        }

        @Override
        public void onAdHidden(MaxAd ad) {
            if (delegate.get() != null) {
                delegate.get().manager.onAdDismissedFullScreenContent(delegate.get().adId);
            }
        }

        @Override
        public void onAdClicked(MaxAd ad) {
            if (delegate.get() != null) {
                delegate.get().manager.onAdClicked(delegate.get().adId);
            }
        }

        @Override
        public void onAdLoadFailed(String adUnitId, MaxError error) {
            if (delegate.get() != null) {
                delegate.get().onAdFailedToLoad(new LoadAdError(error.getCode(), error.getMessage(), error.getMediatedNetworkErrorMessage(), null, null));
            }
        }

        @Override
        public void onAdDisplayFailed(MaxAd ad, MaxError error) {
            if (delegate.get() != null) {
                delegate.get().manager.onFailedToShowFullScreenContent(delegate.get().adId, new AdError(error.getCode(), error.getMessage(), error.getMediatedNetworkErrorMessage()));
            }
        }
    }
}
