// Copyright 2022 Bitcoding
// See the License for the specific language governing permissions and
// limitations under the License.

package com.btcpiyush.ads.google_applovin_unity_ads;

import android.graphics.Bitmap;
import android.graphics.drawable.Drawable;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.btcpiyush.ads.google_applovin_unity_ads.FlutterPaidEventListener;
import com.btcpiyush.ads.google_applovin_unity_ads.FlutterPlatformView;
import com.google.android.gms.ads.AdView;

import io.flutter.plugin.platform.PlatformView;
import io.flutter.util.Preconditions;

/**
 * A wrapper for {@link AdView}.
 */
class FlutterBannerAd extends FlutterAd implements FlutterAdLoadedListener {

    @NonNull
    private final AdInstanceManager manager;
    @NonNull
    private final String adUnitId;
    @NonNull
    private final FlutterAdSize size;
    @NonNull
    private final FlutterAdRequest request;
    @NonNull
    private final BannerAdCreator bannerAdCreator;
    @Nullable
    private AdView adView;

    /**
     * Constructs the FlutterBannerAd.
     */
    public FlutterBannerAd(
            int adId,
            @NonNull AdInstanceManager manager,
            @NonNull String adUnitId,
            @NonNull FlutterAdRequest request,
            @NonNull FlutterAdSize size,
            @NonNull BannerAdCreator bannerAdCreator) {
        super(adId);
        Preconditions.checkNotNull(manager);
        Preconditions.checkNotNull(adUnitId);
        Preconditions.checkNotNull(request);
        Preconditions.checkNotNull(size);
        this.manager = manager;
        this.adUnitId = adUnitId;
        this.request = request;
        this.size = size;
        this.bannerAdCreator = bannerAdCreator;
    }

    @Override
    public void onAdLoaded() {
        if (adView != null) {
            manager.onAdLoaded(adId, adView.getResponseInfo());
        }
    }

    @Override
    void load() {
        adView = bannerAdCreator.createAdView();
        adView.setAdUnitId(adUnitId);
        adView.setAdSize(size.getAdSize());
        adView.setOnPaidEventListener(new FlutterPaidEventListener(manager, this));
        adView.setAdListener(new FlutterBannerAdListener(adId, manager, this));
        adView.loadAd(request.asAdRequest(adUnitId));
    }

    @Nullable
    @Override
    public PlatformView getPlatformView() {
        if (adView == null) {
            return null;
        }
        return new FlutterPlatformView(adView);
    }

    @Override
    void dispose() {
        if (adView != null) {
            adView.destroy();
            adView = null;
        }
    }

    @Nullable
    FlutterAdSize getAdSize() {
        if (adView == null || adView.getAdSize() == null) {
            return null;
        }
        return new FlutterAdSize(adView.getAdSize());
    }

    @Override
    public void onAdLoaded(@NonNull Drawable bitmap) {

    }

    @Override
    public void onAdLoadFailed() {

    }
}
