// Copyright 2022 Bitcoding
// See the License for the specific language governing permissions and
// limitations under the License.

package com.btcpiyush.ads.google_applovin_unity_ads;

import android.graphics.Bitmap;
import android.graphics.drawable.Drawable;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.btcpiyush.ads.google_applovin_unity_ads.databinding.NativeCustomAdsBinding;
import com.bumptech.glide.Glide;
import com.bumptech.glide.request.target.CustomTarget;
import com.bumptech.glide.request.transition.Transition;
import com.facebook.ads.AdView;
import com.google.android.gms.ads.LoadAdError;
import com.squareup.picasso.Picasso;
import com.squareup.picasso.Target;

import io.flutter.plugin.platform.PlatformView;
import io.flutter.util.Preconditions;

/**
 * A wrapper for {@link AdView}.
 */
class FlutterCustomBannerAd extends FlutterAd implements FlutterAdLoadedListener {

    @NonNull
    private final AdInstanceManager manager;
    @NonNull
    private final String adUnitId;
    @NonNull
    private final FlutterAdSize size;
    @NonNull
    private final BannerAdCreator bannerAdCreator;
    @Nullable
    private final NativeCustomAdsBinding adView;
    private final String TAG = "FlutterCustomBannerAd";
    /**
     * Constructs the FlutterBannerAd.
     */
    public FlutterCustomBannerAd(
            int adId,
            @NonNull AdInstanceManager manager,
            @NonNull String adUnitId,
            @NonNull FlutterAdSize size,
            @NonNull BannerAdCreator bannerAdCreator) {
        super(adId);
        android.util.Log.e(TAG, "onMethodCall: : -= -- - - -- -- --          loadCustomBannerAd   ()" );
        Preconditions.checkNotNull(manager);
        Preconditions.checkNotNull(adUnitId);
        Preconditions.checkNotNull(size);
        this.manager = manager;
        this.adUnitId = adUnitId;
        this.size = size;
        this.bannerAdCreator = bannerAdCreator;
        adView = NativeCustomAdsBinding.inflate(LayoutInflater.from(manager.getActivity()));
    }

    @Override
    public void onAdLoaded() {}


    @Override
    public void onAdLoaded(Drawable bitmap) {
        adView.ivNativeImage.setImageDrawable(bitmap);
        manager.onAdLoaded(adId, null);
    }

    int dpToPx(int dp) {
        try {
            return Integer.parseInt(String.valueOf(dp * manager.getActivity().getResources().getSystem().getDisplayMetrics().density));
        } catch (NumberFormatException e) {
            return dp;
        }
    }

    @Override
    void load() {
        Glide.with(manager.getActivity().getApplicationContext()).load(adUnitId).into(new CustomTarget<Drawable>() {
            @Override
            public void onResourceReady(@NonNull Drawable resource, @Nullable Transition<? super Drawable> transition) {
                onAdLoaded(resource);
            }

            @Override
            public void onLoadCleared(@Nullable Drawable placeholder) {}

            @Override
            public void onLoadFailed(@Nullable Drawable errorDrawable) {
                super.onLoadFailed(errorDrawable);
                onAdLoadFailed();
            }
        });

        adView.ivNativeImage.setOnClickListener(view -> manager.onAdClicked(adId));
    }

    @Nullable
    @Override
    public PlatformView getPlatformView() {
        if (adView == null) {
            return null;
        }
        android.util.Log.e(TAG, "onMethodCall: : -= -- - - -- -- --          loadCustomBannerAd   getPlatformView" );
        return new FlutterPlatformView(adView.getRoot());
    }

    @Override
    void dispose() {
    }

    @Nullable
    FlutterAdSize getAdSize() {
       return size;
    }

    @Override
    public void onAdLoadFailed() {
        manager.onAdFailedToLoad(adId, new FlutterAd.FlutterLoadAdError(new LoadAdError(101, "Customer add Loading Failed", "quera",null, null)));
    }
}
