// Copyright 2023 Bitcoding
// See the License for the specific language governing permissions and
// limitations under the License.

package com.btcpiyush.ads.google_applovin_unity_ads;

import android.app.Activity;
import android.app.Dialog;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.drawable.ColorDrawable;
import android.graphics.drawable.Drawable;
import android.net.Uri;
import android.os.Bundle;
import android.transition.Transition;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.WindowManager;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.btcpiyush.ads.google_applovin_unity_ads.databinding.CustomInterstitialAdsBinding;
import com.bumptech.glide.Glide;
import com.facebook.ads.Ad;
import com.google.android.gms.ads.LoadAdError;
import com.squareup.picasso.Picasso;
import com.squareup.picasso.Target;

import java.lang.ref.WeakReference;

class FlutterCustomInterstitialAd extends FlutterAd.FlutterOverlayAd {
    private static final String TAG = "FlutterInterstitialAd";
    @NonNull
    private final AdInstanceManager manager;
    @NonNull
    private final String adUnitId;
    @Nullable
    private CustomAdDialog ad;
    @NonNull
    private final FlutterAdLoader flutterAdLoader;

    public FlutterCustomInterstitialAd(
            int adId,
            @NonNull AdInstanceManager manager,
            @NonNull String adUnitId,
            @NonNull FlutterAdLoader flutterAdLoader) {
        super(adId);
        this.manager = manager;
        this.adUnitId = adUnitId;
        this.flutterAdLoader = flutterAdLoader;
    }

    @Override
    void load() {
        if (manager != null && adUnitId != null) {
            ad = new CustomAdDialog(manager.getActivity(), adUnitId, new DelegatingInterstitialAdLoadCallback(this));
            ad.setCancelable(false);
            ad.setCanceledOnTouchOutside(false);
            ad.loadAd();
        }
    }

    void onAdLoaded(Bitmap bitmap) {
        ad.setBitmap(bitmap);
        manager.onAdLoaded(adId, null);
    }

    void onAdFailedToLoad(LoadAdError loadAdError) {
        ad = null;
        manager.onAdFailedToLoad(adId, new FlutterLoadAdError(loadAdError));
    }

    @Override
    void dispose() {
        if (ad != null)
           ad = null;
    }

    private void onFailedToShow() {
        manager.onFailedToShowFullScreenContent(adId, new com.google.android.gms.ads.AdError(121, "Error showing interstitial - the interstitial ad wasn't loaded yet.","121"));
    }

    @Override
    public void show() {
        if (ad == null) {
            onFailedToShow();
            Log.e(TAG, "Error showing interstitial - the interstitial ad wasn't loaded yet.");
            return;
        }
        ad.show();
    }

    @Override
    public void setImmersiveMode(boolean immersiveModeEnabled) {

    }

    private static final class CustomAdDialog extends Dialog{
        private CustomInterstitialAdsBinding dialogbinding = null;
        final Activity act;
        final String url;
        Bitmap bitmap;
        final DelegatingInterstitialAdLoadCallback listerner;

        CustomAdDialog(
                @NonNull Activity act,
                @NonNull String url,
                @NonNull DelegatingInterstitialAdLoadCallback listerner
        ) {
            super(act, android.R.style.Theme_Black_NoTitleBar_Fullscreen);
            this.act = act;
            this.url = url;
            this.listerner = listerner;
        }


        @Override
        public void show() {
            super.show();
            if (dialogbinding != null && bitmap != null) {
                dialogbinding.ivInterstitialImage.setImageBitmap(bitmap);
                listerner.onInterstitialDisplayed();
                return;
            }
            listerner.onFailedToShowFullScreenContent();
        }

        @Override
        protected void onCreate(Bundle savedInstanceState) {
            super.onCreate(savedInstanceState);
            dialogbinding = CustomInterstitialAdsBinding.inflate(LayoutInflater.from(act));
            if (dialogbinding != null) {
                setContentView(dialogbinding.getRoot());
                dialogbinding.btnClose.setOnClickListener(view -> {
                    dismiss();
                    listerner.onInterstitialDismissed();
                });
                dialogbinding.progressbar.setVisibility(View.GONE);
                dialogbinding.ivInterstitialImage.setVisibility(View.VISIBLE);
                dialogbinding.viewCustomIntra.setOnClickListener(view -> {
                    listerner.onAdClicked();
                });
                dialogbinding.ivInterstitialImage.setOnClickListener(view -> {
                    listerner.onAdClicked();
                });
            }
            setCanceledOnTouchOutside(false);
            WindowManager.LayoutParams layoutParams = new WindowManager.LayoutParams();
            layoutParams.copyFrom(getWindow().getAttributes());
            layoutParams.width = WindowManager.LayoutParams.MATCH_PARENT;
            layoutParams.height = WindowManager.LayoutParams.MATCH_PARENT;
            getWindow().setAttributes(layoutParams);
            getWindow().setBackgroundDrawable(new ColorDrawable(android.graphics.Color.TRANSPARENT));
        }

        public void loadAd() {
            Picasso.get().load(url).into(listerner);
        }

        public void setBitmap(Bitmap bitmap) {
            this.bitmap = bitmap;
            Log.e(TAG, "onMethodCall:    loadCustomInterstitialAd  -------- 444444" );
        }

        /*private void openPage() {
            try {
                Intent browserIntent = new Intent(Intent.ACTION_VIEW, Uri.parse(qurekaLink));
                browserIntent.setPackage("com.android.chrome");
                act.startActivity(browserIntent);
            } catch (Exception exp) {
                try {
                    Intent browserIntent =
                            new Intent(Intent.ACTION_VIEW, Uri.parse(qurekaLink));
                    act.startActivity(browserIntent);
                } catch (Exception ex) {}
            }
        }*/
    }

    /**
     * An InterstitialAdLoadCallback that just forwards events to a delegate.
     */
    private static final class DelegatingInterstitialAdLoadCallback
            implements Target {

        private final WeakReference<FlutterCustomInterstitialAd> delegate;

        DelegatingInterstitialAdLoadCallback(FlutterCustomInterstitialAd delegate) {
            this.delegate = new WeakReference<>(delegate);
        }

        @Override
        public void onBitmapLoaded(Bitmap bitmap, Picasso.LoadedFrom from) {
            if (delegate.get() != null) {
                delegate.get().onAdLoaded(bitmap);
            }
        }


        @Override
        public void onBitmapFailed(Exception e, Drawable errorDrawable) {
            if (delegate.get() != null) {
                delegate.get().onAdFailedToLoad(new LoadAdError(101, e.getMessage(), "quereaka.com", null, null));
            }
        }

        @Override
        public void onPrepareLoad(Drawable placeHolderDrawable) {

        }

        public void onInterstitialDisplayed() {
            if (delegate.get() != null) {
                delegate.get().manager.onAdShowedFullScreenContent(delegate.get().adId);
            }
        }
        public void onFailedToShowFullScreenContent() {
            if (delegate.get() != null) {
                delegate.get().onFailedToShow();
            }
        }

        public void onInterstitialDismissed() {
            if (delegate.get() != null) {
                delegate.get().manager.onAdDismissedFullScreenContent(delegate.get().adId);
            }
        }

        public void onAdClicked() {
            if (delegate.get() != null) {
                delegate.get().manager.onAdClicked(delegate.get().adId);
            }
        }
    }
}
