// Copyright 2022 Bitcoding
// See the License for the specific language governing permissions and
// limitations under the License.

package com.btcpiyush.ads.google_applovin_unity_ads;

import android.graphics.Bitmap;
import android.graphics.drawable.Drawable;
import android.util.Log;
import android.view.LayoutInflater;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import com.btcpiyush.ads.google_applovin_unity_ads.databinding.NativeCustomAdsBinding;
import com.bumptech.glide.Glide;
import com.google.android.gms.ads.LoadAdError;
import com.squareup.picasso.Picasso;

import io.flutter.plugin.platform.PlatformView;

/**
         * A wrapper for {@link NativeAd}.
 */
class FlutterCustomNativeAd extends FlutterAd {
    private static final String TAG = "FlutterNativeAd";

    @NonNull
    private final AdInstanceManager manager;
    @NonNull
    private final String adUnitId;
    private final NativeCustomAdsBinding nativeAd;
    @NonNull
    private final FlutterAdLoader flutterAdLoader;
    @NonNull
    private Boolean isMediumAds;

    static class Builder {
        @Nullable
        private AdInstanceManager manager;
        @Nullable
        private String adUnitId;
        @NonNull
        private Boolean isMediumAds;
        @Nullable
        private Integer id;
        @Nullable
        private FlutterAdLoader flutterAdLoader;

        public Builder setId(int id) {
            this.id = id;
            return this;
        }

        public Builder setManager(@NonNull AdInstanceManager manager) {
            this.manager = manager;
            return this;
        }

        public Builder setAdUnitId(@NonNull String adUnitId) {
            this.adUnitId = adUnitId;
            return this;
        }

        public Builder IsMediumNativeAds(@NonNull Boolean isMediumAds) {
            this.isMediumAds = isMediumAds;
            return this;
        }

        public Builder setFlutterAdLoader(@NonNull FlutterAdLoader flutterAdLoader) {
            this.flutterAdLoader = flutterAdLoader;
            return this;
        }

        FlutterCustomNativeAd build() {
            if (manager == null) {
                throw new IllegalStateException("AdInstanceManager cannot be null.");
            } else if (adUnitId == null) {
                throw new IllegalStateException("AdUnitId cannot be null.");
            }

            if (isMediumAds == null)
                isMediumAds = false;

            final FlutterCustomNativeAd nativeAd =
                        new FlutterCustomNativeAd(
                                id,
                                manager,
                                adUnitId,
                                isMediumAds,
                                flutterAdLoader);

            return nativeAd;
        }
    }

    FlutterCustomNativeAd(
            int adId,
            @NonNull AdInstanceManager manager,
            @NonNull String adUnitId,
            @NonNull Boolean isMediumAds,
            @NonNull FlutterAdLoader flutterAdLoader) {
        super(adId);
        this.manager = manager;
        this.adUnitId = adUnitId;
        this.isMediumAds = isMediumAds;
        this.flutterAdLoader = flutterAdLoader;
        this.nativeAd = NativeCustomAdsBinding.inflate(LayoutInflater.from(manager.getActivity()));
    }

    @Override
    void load() {
        final FlutterCustomNativeAdLoadedListener loadedListener = new FlutterCustomNativeAdLoadedListener(this);
        Glide.with(manager.getActivity().getApplicationContext()).load(adUnitId).into(loadedListener);
    }

    @Override
    @Nullable
    public PlatformView getPlatformView() {
        if (nativeAd == null) {
            return null;
        }
        return new FlutterPlatformView(nativeAd.getRoot());
    }

    public void onNativeAdFailedToLoad() {
        manager.onAdFailedToLoad(adId, new FlutterAd.FlutterLoadAdError(new LoadAdError(101, "NativeAd Loaded Error", "facebook.com",null, null)));
    }


    public void onNativeAdLoaded(Drawable bm) {
        nativeAd.ivNativeImage.setImageDrawable(bm);
        nativeAd.ivNativeImage.setOnClickListener(view -> {
            manager.onAdClicked(adId);
        });
        manager.onAdLoaded(adId, null);
    }

    @Override
    void dispose() {
        Log.e(TAG, "onMethodCall:  ass daksd kas dkasdk ask d 666666666" );
    }
}
