// Copyright 2022 Bitcoding
// See the License for the specific language governing permissions and
// limitations under the License.

package com.btcpiyush.ads.google_applovin_unity_ads;

import android.util.Log;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.facebook.ads.Ad;
import com.facebook.ads.AdError;
import com.facebook.ads.InterstitialAdListener;
import com.google.android.gms.ads.LoadAdError;
import com.facebook.ads.InterstitialAd;

import java.lang.ref.WeakReference;

class FlutterFacebookInterstitialAd extends FlutterAd.FlutterOverlayAd {
    private static final String TAG = "FlutterInterstitialAd";

    @NonNull
    private final AdInstanceManager manager;
    @NonNull
    private final String adUnitId;
    @Nullable
    private InterstitialAd ad;
    @NonNull
    private final FlutterAdLoader flutterAdLoader;

    public FlutterFacebookInterstitialAd(
            int adId,
            @NonNull AdInstanceManager manager,
            @NonNull String adUnitId,
            @NonNull FlutterAdLoader flutterAdLoader) {
        super(adId);
        this.manager = manager;
        this.adUnitId = adUnitId;
        this.flutterAdLoader = flutterAdLoader;
    }

    @Override
    void load() {
        if (manager != null && adUnitId != null) {
            ad = flutterAdLoader.loadFacebookInterstitial(
                    manager.getActivity(),
                    adUnitId, new DelegatingInterstitialAdLoadCallback(this));
        }
    }

    void onAdLoaded() {
        manager.onAdLoaded(adId, null);
    }

    void onAdFailedToLoad(LoadAdError loadAdError) {
        manager.onAdFailedToLoad(adId, new FlutterLoadAdError(loadAdError));
    }

    @Override
    void dispose() {
        if (ad != null)
            ad.destroy();
        
        ad = null;
    }

    @Override
    public void show() {
        if (ad == null) {
            Log.e(TAG, "Error showing interstitial - the interstitial ad wasn't loaded yet.");
            return;
        }
        ad.show();
    }

    @Override
    public void setImmersiveMode(boolean immersiveModeEnabled) {

    }

    /**
     * An InterstitialAdLoadCallback that just forwards events to a delegate.
     */
    private static final class DelegatingInterstitialAdLoadCallback
            implements InterstitialAdListener {

        private final WeakReference<FlutterFacebookInterstitialAd> delegate;

        DelegatingInterstitialAdLoadCallback(FlutterFacebookInterstitialAd delegate) {
            this.delegate = new WeakReference<>(delegate);
        }

        @Override
        public void onInterstitialDisplayed(Ad ad) {
            if (delegate.get() != null) {
                delegate.get().manager.onAdShowedFullScreenContent(delegate.get().adId);
            }
        }

        @Override
        public void onInterstitialDismissed(Ad ad) {
            if (delegate.get() != null) {
                delegate.get().manager.onAdDismissedFullScreenContent(delegate.get().adId);
            }
        }

        @Override
        public void onError(Ad ad, AdError adError) {
            if (delegate.get() != null) {
                delegate.get().onAdFailedToLoad(new LoadAdError(adError.getErrorCode(), adError.getErrorMessage(), "facebook.com", null, null));
            }
        }

        @Override
        public void onAdLoaded(Ad ad) {
            if (delegate.get() != null) {
                delegate.get().onAdLoaded();
            }
        }

        @Override
        public void onAdClicked(Ad ad) {
            if (delegate.get() != null) {
                delegate.get().manager.onAdClicked(delegate.get().adId);
            }
        }

        @Override
        public void onLoggingImpression(Ad ad) {
            if (delegate.get() != null) {
                delegate.get().manager.onAdImpression(delegate.get().adId);
            }
        }
    }
}
