// Copyright 2022 Bitcoding
// See the License for the specific language governing permissions and
// limitations under the License.

package com.btcpiyush.ads.google_applovin_unity_ads;

import android.util.Log;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.btcpiyush.ads.google_applovin_unity_ads.GoogleApplovinUnityAdsPlugin.NativeAdFactory;
import com.facebook.ads.AdError;
import com.facebook.ads.NativeAd;
import com.facebook.ads.NativeAdLayout;
import com.facebook.ads.NativeAdListener;
import com.facebook.ads.NativeAdView;
import com.google.android.gms.ads.LoadAdError;

import java.util.Map;

import io.flutter.plugin.platform.PlatformView;

/**
 * A wrapper for {@link NativeAd}.
 */
class FlutterFacebookNativeAd extends FlutterAd {
    private static final String TAG = "FlutterNativeAd";

    @NonNull
    private final AdInstanceManager manager;
    @NonNull
    private final String adUnitId;
    @NonNull
    private final NativeAdFactory adFactory;
    @NonNull
    private final FlutterAdLoader flutterAdLoader;
    @NonNull
    private Boolean isMediumAds;
    @NonNull
    private NativeAd nativeAd;
    @Nullable
    private NativeAdLayout nativeAdView;

    static class Builder {
        @Nullable
        private AdInstanceManager manager;
        @Nullable
        private String adUnitId;
        @Nullable
        private NativeAdFactory adFactory;
        @NonNull
        private Boolean isMediumAds;
        @Nullable
        private Integer id;
        @Nullable
        private FlutterAdLoader flutterAdLoader;

        public Builder setAdFactory(@NonNull NativeAdFactory adFactory) {
            this.adFactory = adFactory;
            return this;
        }

        public Builder setId(int id) {
            this.id = id;
            return this;
        }

        public Builder setManager(@NonNull AdInstanceManager manager) {
            this.manager = manager;
            return this;
        }

        public Builder setAdUnitId(@NonNull String adUnitId) {
            this.adUnitId = adUnitId;
            return this;
        }

        public Builder IsMediumNativeAds(@NonNull Boolean isMediumAds) {
            this.isMediumAds = isMediumAds;
            return this;
        }

        public Builder setFlutterAdLoader(@NonNull FlutterAdLoader flutterAdLoader) {
            this.flutterAdLoader = flutterAdLoader;
            return this;
        }

        FlutterFacebookNativeAd build() {
            if (manager == null) {
                throw new IllegalStateException("AdInstanceManager cannot be null.");
            } else if (adUnitId == null) {
                throw new IllegalStateException("AdUnitId cannot be null.");
            } else if (adFactory == null) {
                throw new IllegalStateException("NativeAdFactory cannot be null.");
            }

            if (isMediumAds == null)
                isMediumAds = false;

            final FlutterFacebookNativeAd nativeAd =
                        new FlutterFacebookNativeAd(
                                id,
                                manager,
                                adUnitId,
                                isMediumAds,
                                adFactory,
                                flutterAdLoader);

            return nativeAd;
        }
    }

    FlutterFacebookNativeAd(
            int adId,
            @NonNull AdInstanceManager manager,
            @NonNull String adUnitId,
            @NonNull Boolean isMediumAds,
            @NonNull NativeAdFactory adFactory,
            @NonNull FlutterAdLoader flutterAdLoader) {
        super(adId);
        this.manager = manager;
        this.adUnitId = adUnitId;
        this.adFactory = adFactory;
        this.isMediumAds = isMediumAds;
        this.flutterAdLoader = flutterAdLoader;
    }

    @Override
    void load() {
        final NativeAdListener loadedListener = new FlutterFacebookNativeAdLoadedListener(this);
        nativeAd = flutterAdLoader.loadFacebookNativeAd(adUnitId);
        nativeAd.loadAd(
                nativeAd.buildLoadAdConfig()
                .withAdListener(loadedListener).build());
    }

    @Override
    @Nullable
    public PlatformView getPlatformView() {
        if (nativeAdView == null) {
            return null;
        }
        return new FlutterPlatformView(nativeAdView);
    }

    void onNativeAdFailedToLoad(AdError error) {
        manager.onAdFailedToLoad(adId, new FlutterAd.FlutterLoadAdError(new LoadAdError(error.getErrorCode(), error.getErrorMessage(), "facebook.com",null, null)));
    }

    void onNativeAdClicked() {
        manager.onAdClicked(adId);
    }


    void onLoggingImpression() {
        manager.onAdImpression(adId);
    }


    void onNativeAdLoaded() {
        nativeAdView = isMediumAds ? adFactory.createFacebookMediumNativeAd(manager.getActivity(), nativeAd) : adFactory.createFacebookNativeAd(manager.getActivity(), nativeAd);
        manager.onAdLoaded(adId, null);
    }

    @Override
    void dispose() {
        if (nativeAd != null) {
            nativeAd.destroy();
            nativeAd = null;
        }

        if (nativeAdView != null) {
            nativeAdView = null;
        }
    }
}
