// Copyright 2022 Bitcoding
// See the License for the specific language governing permissions and
// limitations under the License.

package com.btcpiyush.ads.google_applovin_unity_ads;

import android.util.Log;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import com.google.android.gms.ads.AdListener;
import com.google.android.gms.ads.admanager.AdManagerAdRequest;
import com.google.android.gms.ads.nativead.NativeAd;
import com.google.android.gms.ads.nativead.NativeAd.OnNativeAdLoadedListener;
import com.google.android.gms.ads.nativead.NativeAdOptions;
import com.google.android.gms.ads.nativead.NativeAdView;

import java.util.Map;

import io.flutter.plugin.platform.PlatformView;

import com.btcpiyush.ads.google_applovin_unity_ads.GoogleApplovinUnityAdsPlugin.NativeAdFactory;

/**
 * A wrapper for {@link NativeAd}.
 */
class FlutterNativeAd extends FlutterAd {
    private static final String TAG = "FlutterNativeAd";

    @NonNull
    private final AdInstanceManager manager;
    @NonNull
    private final String adUnitId;
    @NonNull
    private final NativeAdFactory adFactory;
    @NonNull
    private final FlutterAdLoader flutterAdLoader;
    @Nullable
    private FlutterAdRequest request;
    @NonNull
    private Boolean isMediumAds;
    @Nullable
    private FlutterAdManagerAdRequest adManagerRequest;
    @Nullable
    private Map<String, Object> customOptions;
    @Nullable
    private NativeAdView nativeAdView;
    @Nullable
    private final FlutterNativeAdOptions nativeAdOptions;

    static class Builder {
        @Nullable
        private AdInstanceManager manager;
        @Nullable
        private String adUnitId;
        @Nullable
        private NativeAdFactory adFactory;
        @Nullable
        private FlutterAdRequest request;
        @NonNull
        private Boolean isMediumAds;
        @Nullable
        private FlutterAdManagerAdRequest adManagerRequest;
        @Nullable
        private Map<String, Object> customOptions;
        @Nullable
        private Integer id;
        @Nullable
        private FlutterNativeAdOptions nativeAdOptions;
        @Nullable
        private FlutterAdLoader flutterAdLoader;

        public Builder setAdFactory(@NonNull NativeAdFactory adFactory) {
            this.adFactory = adFactory;
            return this;
        }

        public Builder setId(int id) {
            this.id = id;
            return this;
        }

        public Builder setCustomOptions(@Nullable Map<String, Object> customOptions) {
            this.customOptions = customOptions;
            return this;
        }

        public Builder setManager(@NonNull AdInstanceManager manager) {
            this.manager = manager;
            return this;
        }

        public Builder setAdUnitId(@NonNull String adUnitId) {
            this.adUnitId = adUnitId;
            return this;
        }

        public Builder IsMediumNativeAds(@NonNull Boolean isMediumAds) {
            this.isMediumAds = isMediumAds;
            return this;
        }

        public Builder setRequest(@NonNull FlutterAdRequest request) {
            this.request = request;
            return this;
        }

        public Builder setAdManagerRequest(@NonNull FlutterAdManagerAdRequest request) {
            this.adManagerRequest = request;
            return this;
        }

        public Builder setNativeAdOptions(@Nullable FlutterNativeAdOptions nativeAdOptions) {
            this.nativeAdOptions = nativeAdOptions;
            return this;
        }

        public Builder setFlutterAdLoader(@NonNull FlutterAdLoader flutterAdLoader) {
            this.flutterAdLoader = flutterAdLoader;
            return this;
        }

        FlutterNativeAd build() {
            if (manager == null) {
                throw new IllegalStateException("AdInstanceManager cannot be null.");
            } else if (adUnitId == null) {
                throw new IllegalStateException("AdUnitId cannot be null.");
            } else if (adFactory == null) {
                throw new IllegalStateException("NativeAdFactory cannot be null.");
            } else if (request == null && adManagerRequest == null) {
                throw new IllegalStateException("adRequest or addManagerRequest must be non-null.");
            }

            if (isMediumAds == null)
                isMediumAds = false;

            final FlutterNativeAd nativeAd;
            if (request == null) {
                nativeAd =
                        new FlutterNativeAd(
                                id,
                                manager,
                                adUnitId,
                                isMediumAds,
                                adFactory,
                                adManagerRequest,
                                flutterAdLoader,
                                customOptions,
                                nativeAdOptions);
            } else {
                nativeAd =
                        new FlutterNativeAd(
                                id,
                                manager,
                                adUnitId,
                                isMediumAds,
                                adFactory,
                                request,
                                flutterAdLoader,
                                customOptions,
                                nativeAdOptions);
            }
            return nativeAd;
        }
    }

    FlutterNativeAd(
            int adId,
            @NonNull AdInstanceManager manager,
            @NonNull String adUnitId,
            @NonNull Boolean isMediumAds,
            @NonNull NativeAdFactory adFactory,
            @NonNull FlutterAdRequest request,
            @NonNull FlutterAdLoader flutterAdLoader,
            @Nullable Map<String, Object> customOptions,
            @Nullable FlutterNativeAdOptions nativeAdOptions) {
        super(adId);
        this.manager = manager;
        this.adUnitId = adUnitId;
        this.adFactory = adFactory;
        this.request = request;
        this.isMediumAds = isMediumAds;
        this.flutterAdLoader = flutterAdLoader;
        this.customOptions = customOptions;
        this.nativeAdOptions = nativeAdOptions;
    }

    FlutterNativeAd(
            int adId,
            @NonNull AdInstanceManager manager,
            @NonNull String adUnitId,
            @NonNull Boolean isMediumAds,
            @NonNull NativeAdFactory adFactory,
            @NonNull FlutterAdManagerAdRequest adManagerRequest,
            @NonNull FlutterAdLoader flutterAdLoader,
            @Nullable Map<String, Object> customOptions,
            @Nullable FlutterNativeAdOptions nativeAdOptions) {
        super(adId);
        this.manager = manager;
        this.adUnitId = adUnitId;
        this.adFactory = adFactory;
        this.isMediumAds = isMediumAds;
        this.adManagerRequest = adManagerRequest;
        this.flutterAdLoader = flutterAdLoader;
        this.customOptions = customOptions;
        this.nativeAdOptions = nativeAdOptions;
    }

    @Override
    void load() {
        final OnNativeAdLoadedListener loadedListener = new FlutterNativeAdLoadedListener(this);
        final AdListener adListener = new FlutterNativeAdListener(adId, manager);
        // Note we delegate loading the ad to FlutterAdLoader mainly for testing purposes.
        // As of 20.0.0 of GMA, mockito is unable to mock AdLoader.
        final NativeAdOptions options =
                this.nativeAdOptions == null
                        ? new NativeAdOptions.Builder().build()
                        : nativeAdOptions.asNativeAdOptions();
        if (request != null) {
            flutterAdLoader.loadNativeAd(
                    adUnitId, loadedListener, options, adListener, request.asAdRequest(adUnitId));
        } else if (adManagerRequest != null) {
            AdManagerAdRequest adManagerAdRequest = adManagerRequest.asAdManagerAdRequest(adUnitId);
            flutterAdLoader.loadAdManagerNativeAd(
                    adUnitId, loadedListener, options, adListener, adManagerAdRequest);
        } else {
            Log.e(TAG, "A null or invalid ad request was provided.");
        }
    }

    @Override
    @Nullable
    public PlatformView getPlatformView() {
        if (nativeAdView == null) {
            return null;
        }
        return new FlutterPlatformView(nativeAdView);
    }

    void onNativeAdLoaded(@NonNull NativeAd nativeAd) {
        nativeAdView = isMediumAds ? adFactory.createMediumNativeAd(nativeAd, customOptions) : adFactory.createNativeAd(nativeAd, customOptions);
        nativeAd.setOnPaidEventListener(new FlutterPaidEventListener(manager, this));
        manager.onAdLoaded(adId, nativeAd.getResponseInfo());
    }

    @Override
    void dispose() {
        if (nativeAdView != null) {
            nativeAdView.destroy();
            nativeAdView = null;
        }
    }
}
