// Copyright 2022 Bitcoding
// See the License for the specific language governing permissions and
// limitations under the License.

package com.btcpiyush.ads.google_applovin_unity_ads;

import android.app.Activity;
import android.content.Context;
import android.util.Log;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.annotation.VisibleForTesting;

import com.applovin.mediation.MaxAd;
import com.applovin.mediation.MaxAdListener;
import com.applovin.mediation.MaxRewardedAdListener;
import com.applovin.mediation.ads.MaxAdView;
import com.applovin.mediation.ads.MaxInterstitialAd;
import com.applovin.mediation.ads.MaxRewardedAd;
import com.applovin.mediation.nativeAds.MaxNativeAdView;
import com.applovin.sdk.AppLovinMediationProvider;
import com.applovin.sdk.AppLovinSdk;
import com.applovin.sdk.AppLovinSdkConfiguration;
import com.facebook.ads.NativeAdLayout;
import com.google.android.gms.ads.AdSize;
import com.google.android.gms.ads.LoadAdError;
import com.google.android.gms.ads.MobileAds;
import com.google.android.gms.ads.RequestConfiguration;
import com.google.android.gms.ads.initialization.InitializationStatus;
import com.google.android.gms.ads.initialization.OnInitializationCompleteListener;
import com.google.android.gms.ads.nativead.NativeAd;
import com.google.android.gms.ads.nativead.NativeAdView;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import io.flutter.embedding.engine.FlutterEngine;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.StandardMethodCodec;

import com.btcpiyush.ads.google_applovin_unity_ads.FlutterAd.FlutterOverlayAd;
import com.btcpiyush.ads.google_applovin_unity_ads.usermessagingplatform.UserMessagingPlatformManager;

/**
 * Flutter plugin accessing Google Mobile Ads API.
 *
 * <p>Instantiate this in an add to app scenario to gracefully handle activity and context changes.
 */
public class GoogleApplovinUnityAdsPlugin implements FlutterPlugin, ActivityAware, MethodCallHandler {

    private static final String TAG = "GoogleMobileAdsPlugin";

    private static <T> T requireNonNull(T obj) {
        if (obj == null) {
            throw new IllegalArgumentException();
        }
        return obj;
    }

    @Nullable
    private FlutterPluginBinding pluginBinding;
    @Nullable
    private AdInstanceManager instanceManager;
    @Nullable
    private AdMessageCodec adMessageCodec;
    @Nullable
    private AppStateNotifier appStateNotifier;
    @Nullable
    private UserMessagingPlatformManager userMessagingPlatformManager;
    private final Map<String, NativeAdFactory> nativeAdFactories = new HashMap<>();
    @Nullable
    private MediationNetworkExtrasProvider mediationNetworkExtrasProvider;
    private final FlutterMobileAdsWrapper flutterMobileAds;
    private final FlutterAppLovinWrapper flutterAppLovinAds;
    private final FlutterFacebookAdsWrapper flutterfacebookAds;

    /**
     * Public constructor for the plugin. Dependency initialization is handled in lifecycle methods
     * below.
     */
    public GoogleApplovinUnityAdsPlugin() {
        this.flutterMobileAds = new FlutterMobileAdsWrapper();
        this.flutterAppLovinAds = new FlutterAppLovinWrapper();
        this.flutterfacebookAds = new FlutterFacebookAdsWrapper();
    }

    /**
     * Constructor for testing.
     */
    @VisibleForTesting
    protected GoogleApplovinUnityAdsPlugin(
            @Nullable FlutterPluginBinding pluginBinding,
            @Nullable AdInstanceManager instanceManager,
            @NonNull FlutterMobileAdsWrapper flutterMobileAds,
            @NonNull FlutterFacebookAdsWrapper flutterfacebookAds,
            @NonNull FlutterAppLovinWrapper flutterAppLovinAds) {
        this.pluginBinding = pluginBinding;
        this.instanceManager = instanceManager;
        this.flutterMobileAds = flutterMobileAds;
        this.flutterfacebookAds = flutterfacebookAds;
        this.flutterAppLovinAds = flutterAppLovinAds;
    }

    @VisibleForTesting
    protected GoogleApplovinUnityAdsPlugin(@NonNull AppStateNotifier appStateNotifier) {
        this.appStateNotifier = appStateNotifier;
        this.flutterMobileAds = new FlutterMobileAdsWrapper();
        this.flutterfacebookAds = new FlutterFacebookAdsWrapper();
        this.flutterAppLovinAds = new FlutterAppLovinWrapper();
    }

    /**
     * Interface used to display a {@link NativeAd}.
     *
     * <p>Added to a {@link GoogleApplovinUnityAdsPlugin} and creates
     * {@link NativeAdView}s from Native Ads created in Dart.
     */
    public interface NativeAdFactory {
        /**
         * Creates a {@link NativeAdView} with a {@link
         * NativeAd}.
         *
         * @param nativeAd      Ad information used to create a {@link
         *                      NativeAd}
         * @param customOptions Used to pass additional custom options to create the {@link
         *                      NativeAdView}. Nullable.
         * @return a {@link NativeAdView} that is overlaid on top of
         * the FlutterView
         */
        NativeAdView createNativeAd(NativeAd nativeAd, Map<String, Object> customOptions);

        NativeAdView createMediumNativeAd(NativeAd nativeAd, Map<String, Object> customOptions);

        NativeAdLayout createFacebookMediumNativeAd(@NonNull Activity context, com.facebook.ads.NativeAd nativeAd);

        NativeAdLayout createFacebookNativeAd(@NonNull Activity context, com.facebook.ads.NativeAd nativeAd);

        MaxNativeAdView createAppLovinNativeAd(@NonNull Activity context);

        MaxNativeAdView createAppLovinMediumNativeAd(@NonNull Activity context);
    }

    /**
     * Registers a {@link MediationNetworkExtrasProvider} used to provide network extras to the plugin
     * when it creates ad requests.
     *
     * @param engine                         The {@link FlutterEngine} which should have an attached instance of this plugin
     * @param mediationNetworkExtrasProvider The {@link MediationNetworkExtrasProvider} which will be
     *                                       used to provide network extras when ad requests are created
     * @return whether {@code mediationNetworkExtrasProvider} was registered to a {@code
     * GoogleMobileAdsPlugin} associated with {@code engine}
     */
    public static boolean registerMediationNetworkExtrasProvider(
            FlutterEngine engine, MediationNetworkExtrasProvider mediationNetworkExtrasProvider) {
        final GoogleApplovinUnityAdsPlugin gmaPlugin =
                (GoogleApplovinUnityAdsPlugin) engine.getPlugins().get(GoogleApplovinUnityAdsPlugin.class);
        if (gmaPlugin == null) {
            return false;
        }
        gmaPlugin.mediationNetworkExtrasProvider = mediationNetworkExtrasProvider;
        if (gmaPlugin.adMessageCodec != null) {
            gmaPlugin.adMessageCodec.setMediationNetworkExtrasProvider(mediationNetworkExtrasProvider);
        }
        return true;
    }

    /**
     * Unregisters any {@link MediationNetworkExtrasProvider} that have been previously registered
     * with the plugin using {@code unregisterMediationNetworkExtrasProvider}.
     *
     * @param engine The {@link FlutterEngine} which should have an attached instance of this plugin
     */
    public static void unregisterMediationNetworkExtrasProvider(FlutterEngine engine) {
        final GoogleApplovinUnityAdsPlugin gmaPlugin =
                (GoogleApplovinUnityAdsPlugin) engine.getPlugins().get(GoogleApplovinUnityAdsPlugin.class);
        if (gmaPlugin == null) {
            return;
        }

        if (gmaPlugin.adMessageCodec != null) {
            gmaPlugin.adMessageCodec.setMediationNetworkExtrasProvider(null);
        }
        gmaPlugin.mediationNetworkExtrasProvider = null;
    }

    /**
     * Registers a {@link NativeAdFactory}
     * used to create {@link NativeAdView}s from a Native Ad
     * created in Dart.
     *
     * @param engine          maintains access to a GoogleMobileAdsPlugin instance
     * @param factoryId       a unique identifier for the ad factory. The Native Ad created in Dart includes
     *                        a parameter that refers to this.
     * @param nativeAdFactory creates {@link NativeAdView}s when
     *                        Flutter NativeAds are created
     * @return whether the factoryId is unique and the nativeAdFactory was successfully added
     */
    public static boolean registerNativeAdFactory(
            FlutterEngine engine, String factoryId, NativeAdFactory nativeAdFactory) {
        final GoogleApplovinUnityAdsPlugin gmaPlugin =
                (GoogleApplovinUnityAdsPlugin) engine.getPlugins().get(GoogleApplovinUnityAdsPlugin.class);
        return registerNativeAdFactory(gmaPlugin, factoryId, nativeAdFactory);
    }


    private static boolean registerNativeAdFactory(
            GoogleApplovinUnityAdsPlugin plugin, String factoryId, NativeAdFactory nativeAdFactory) {
        if (plugin == null) {
            final String message =
                    String.format(
                            "Could not find a %s instance. The plugin may have not been registered.",
                            GoogleApplovinUnityAdsPlugin.class.getSimpleName());
            throw new IllegalStateException(message);
        }

        return plugin.addNativeAdFactory(factoryId, nativeAdFactory);
    }

    /**
     * Unregisters a {@link NativeAdFactory}
     * used to create {@link NativeAdView}s from a Native Ad
     * created in Dart.
     *
     * @param engine    maintains access to a GoogleMobileAdsPlugin instance
     * @param factoryId a unique identifier for the ad factory. The Native ad created in Dart includes
     *                  a parameter that refers to this
     * @return the previous {@link
     * NativeAdFactory} associated with
     * this factoryId, or null if there was none for this factoryId
     */
    @Nullable
    public static NativeAdFactory unregisterNativeAdFactory(FlutterEngine engine, String factoryId) {
        final FlutterPlugin gmaPlugin = engine.getPlugins().get(GoogleApplovinUnityAdsPlugin.class);
        if (gmaPlugin != null) {
            return ((GoogleApplovinUnityAdsPlugin) gmaPlugin).removeNativeAdFactory(factoryId);
        }

        return null;
    }


    private boolean addNativeAdFactory(String factoryId, NativeAdFactory nativeAdFactory) {
        if (nativeAdFactories.containsKey(factoryId)) {
            final String errorMessage =
                    String.format(
                            "A NativeAdFactory with the following factoryId already exists: %s", factoryId);
            Log.e(GoogleApplovinUnityAdsPlugin.class.getSimpleName(), errorMessage);
            return false;
        }

        nativeAdFactories.put(factoryId, nativeAdFactory);
        return true;
    }

    private NativeAdFactory removeNativeAdFactory(String factoryId) {
        return nativeAdFactories.remove(factoryId);
    }

    @Override
    public void onAttachedToEngine(FlutterPluginBinding binding) {
        pluginBinding = binding;
        adMessageCodec = new AdMessageCodec(binding.getApplicationContext());
        if (mediationNetworkExtrasProvider != null) {
            adMessageCodec.setMediationNetworkExtrasProvider(mediationNetworkExtrasProvider);
        }
        final MethodChannel channel =
                new MethodChannel(
                        binding.getBinaryMessenger(),
                        "plugins.flutter.io/google_applovin_unity_ads",
                        new StandardMethodCodec(adMessageCodec));
        channel.setMethodCallHandler(this);
        instanceManager = new AdInstanceManager(channel);
        binding
                .getPlatformViewRegistry()
                .registerViewFactory(
                        "plugins.flutter.io/google_applovin_unity_ads/ad_widget",
                        new GoogleMobileAdsViewFactory(instanceManager));
        appStateNotifier = new AppStateNotifier(binding.getBinaryMessenger());
        userMessagingPlatformManager =
                new UserMessagingPlatformManager(
                        binding.getBinaryMessenger(), binding.getApplicationContext());
    }

    @Override
    public void onDetachedFromEngine(FlutterPluginBinding binding) {
        if (appStateNotifier != null) {
            appStateNotifier.stop();
            appStateNotifier = null;
        }
    }

    @Override
    public void onAttachedToActivity(ActivityPluginBinding binding) {
        if (instanceManager != null) {
            instanceManager.setActivity(binding.getActivity());
        }
        if (adMessageCodec != null) {
            adMessageCodec.setContext(binding.getActivity());
        }
        if (userMessagingPlatformManager != null) {
            userMessagingPlatformManager.setActivity(binding.getActivity());
        }
    }

    @Override
    public void onDetachedFromActivityForConfigChanges() {
        // Use the application context
        if (adMessageCodec != null && pluginBinding != null) {
            adMessageCodec.setContext(pluginBinding.getApplicationContext());
        }
        if (instanceManager != null) {
            instanceManager.setActivity(null);
        }
        if (userMessagingPlatformManager != null) {
            userMessagingPlatformManager.setActivity(null);
        }
    }

    @Override
    public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {
        if (instanceManager != null) {
            instanceManager.setActivity(binding.getActivity());
        }
        if (adMessageCodec != null) {
            adMessageCodec.setContext(binding.getActivity());
        }
        if (userMessagingPlatformManager != null) {
            userMessagingPlatformManager.setActivity(binding.getActivity());
        }
    }

    @Override
    public void onDetachedFromActivity() {
        if (adMessageCodec != null && pluginBinding != null) {
            adMessageCodec.setContext(pluginBinding.getApplicationContext());
        }
        if (instanceManager != null) {
            instanceManager.setActivity(null);
        }
        if (userMessagingPlatformManager != null) {
            userMessagingPlatformManager.setActivity(null);
        }
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull final Result result) {
        if (instanceManager == null || pluginBinding == null) {
            Log.e(TAG, "method call received before instanceManager initialized: " + call.method);
            return;
        }
        // Use activity as context if available.
/*    Context context =
        (instanceManager.getActivity() != null)
            ? instanceManager.getActivity()
            : pluginBinding.getApplicationContext();*/

        Activity context = instanceManager.getActivity();

        switch (call.method) {
            case "_init":
                // Internal init. This is necessary to cleanup state on hot restart.
                instanceManager.disposeAllAds();
                result.success(null);
                break;
            case "MobileAds#initialize":
                flutterMobileAds.initialize(context, new FlutterInitializationListener(result));
                break;
            case "FacebookAds#initialize":
                flutterfacebookAds.initialize(context);
                break;
            case "AppLovinAds#initialize":
                String appLovinSdkKey = call.argument("applovin_sdk_key");
                if (appLovinSdkKey == null) {
                    final String appLovinmessage = String.format("Can't find AppLovin SDK Key with id");
                    result.error("SDKKeyError", appLovinmessage, null);
                    break;
                }

                String geosdkKey = null;
                Integer timeoutInSeconds = 0;
                try {
                    geosdkKey = call.argument("geoedge_sdk_key");
                    timeoutInSeconds = call.argument("timeoutInSeconds");
                } catch (Exception e) {
                } finally {
                    flutterAppLovinAds.initializeSDK(context, appLovinSdkKey);
                    flutterAppLovinAds.setVerboseLogging(false);
                    flutterAppLovinAds.setMediationProvider(AppLovinMediationProvider.MAX);
                    if (geosdkKey != null && !geosdkKey.isEmpty()) {
                        flutterAppLovinAds.initialize(new FlutterAppLovinInitializationListener(result), context, geosdkKey, timeoutInSeconds);
                    } else {
                        flutterAppLovinAds.initialize(new FlutterAppLovinInitializationListener(result), null, null, 0);
                    }
                }
                break;
            case "MobileAds#openAdInspector":
                flutterMobileAds.openAdInspector(
                        context,
                        adInspectorError -> {
                            if (adInspectorError != null) {
                                String errorCode = Integer.toString(adInspectorError.getCode());
                                result.error(
                                        errorCode, adInspectorError.getMessage(), adInspectorError.getDomain());
                            } else {
                                result.success(null);
                            }
                        });
                break;
            case "MobileAds#getRequestConfiguration":
                result.success(flutterMobileAds.getRequestConfiguration());
                break;
            case "AppLovinAds#getRequestConfiguration":
                result.success(flutterAppLovinAds.getRequestConfiguration().toString());
                break;
            case "MobileAds#updateRequestConfiguration":
                RequestConfiguration.Builder builder = MobileAds.getRequestConfiguration().toBuilder();
                String maxAdContentRating = call.argument("maxAdContentRating");
                Integer tagForChildDirectedTreatment = call.argument("tagForChildDirectedTreatment");
                Integer tagForUnderAgeOfConsent = call.argument("tagForUnderAgeOfConsent");
                List<String> testDeviceIds = call.argument("testDeviceIds");
                if (maxAdContentRating != null) {
                    builder.setMaxAdContentRating(maxAdContentRating);
                }
                if (tagForChildDirectedTreatment != null) {
                    builder.setTagForChildDirectedTreatment(tagForChildDirectedTreatment);
                }
                if (tagForUnderAgeOfConsent != null) {
                    builder.setTagForUnderAgeOfConsent(tagForUnderAgeOfConsent);
                }
                if (testDeviceIds != null) {
                    builder.setTestDeviceIds(testDeviceIds);
                }
                MobileAds.setRequestConfiguration(builder.build());
                result.success(null);
                break;
            case "loadBannerAd":
                final FlutterBannerAd bannerAd =
                        new FlutterBannerAd(
                                call.<Integer>argument("adId"),
                                instanceManager,
                                call.<String>argument("adUnitId"),
                                call.<FlutterAdRequest>argument("request"),
                                call.<FlutterAdSize>argument("size"),
                                getBannerAdCreator(context));
                instanceManager.trackAd(bannerAd, call.<Integer>argument("adId"));
                bannerAd.load();
                result.success(null);
                break;
            case "loadFacebookBannerAd":
                final FlutterFacebookBannerAd fbBannerAd =
                        new FlutterFacebookBannerAd(
                                call.<Integer>argument("adId"),
                                instanceManager,
                                call.<String>argument("adUnitId"),
//                                "IMG_16_9_APP_INSTALL#YOUR_PLACEMENT_ID",
                                call.<FlutterAdSize>argument("size"),
                                getBannerAdCreator(context));
                instanceManager.trackAd(fbBannerAd, call.<Integer>argument("adId"));
                fbBannerAd.load();
                result.success(null);
                break;
            case "loadCustomBannerAd":
                android.util.Log.e(TAG, "onMethodCall: : -= -- - - -- -- --          loadCustomBannerAd" );
                final FlutterCustomBannerAd custBannerAd =
                        new FlutterCustomBannerAd(
                                call.<Integer>argument("adId"),
                                instanceManager,
                                call.<String>argument("adUnitId"),
                                call.<FlutterAdSize>argument("size"),
                                getBannerAdCreator(context));
                instanceManager.trackAd(custBannerAd, call.<Integer>argument("adId"));
                custBannerAd.load();
                result.success(null);
                break;
            case "loadAppLovinBannerAd":
                if (flutterAppLovinAds.getAppLovinSdk() != null) {
                    final FlutterAppLovinBannerAd appLovininterstitial = new FlutterAppLovinBannerAd(
                            call.<Integer>argument("adId"),
                            instanceManager,
                            call.<String>argument("adUnitId"),
                            call.<FlutterAdSize>argument("size"),
                            flutterAppLovinAds.getAppLovinSdk(),
                            getBannerAdCreator(context));
                    instanceManager.trackAd(appLovininterstitial, call.<Integer>argument("adId"));
                    appLovininterstitial.load();
                    result.success(null);
                } else {
                    instanceManager.onAdFailedToLoad(call.<Integer>argument("adId"), new FlutterAd.FlutterLoadAdError(new LoadAdError(999, "AppLovin SDK not Intitalized", "applovin.com", null, null)));
                    result.success(null);
//                    result.error("InvalidRequest", , null);
                }
                break;
            case "loadNativeAd":
                final String factoryId = call.argument("factoryId");
                final NativeAdFactory factory = nativeAdFactories.get(factoryId);
                if (factory == null) {
                    final String message = String.format("Can't find NativeAdFactory with id: %s", factoryId);
                    result.error("NativeAdError", message, null);
                    break;
                }

                Boolean isMediumNativeAds = call.<Boolean>argument("isMediumNativeAds");
                isMediumNativeAds = isMediumNativeAds == null ? false :isMediumNativeAds;
                final FlutterNativeAd nativeAd =
                        new FlutterNativeAd.Builder()
                                .setManager(instanceManager)
                                .setAdUnitId(call.<String>argument("adUnitId"))
                                .setAdFactory(factory)
                                .IsMediumNativeAds(isMediumNativeAds)
                                .setRequest(call.<FlutterAdRequest>argument("request"))
                                .setAdManagerRequest(call.<FlutterAdManagerAdRequest>argument("adManagerRequest"))
                                .setCustomOptions(call.<Map<String, Object>>argument("customOptions"))
                                .setId(call.<Integer>argument("adId"))
                                .setNativeAdOptions(call.<FlutterNativeAdOptions>argument("nativeAdOptions"))
                                .setFlutterAdLoader(new FlutterAdLoader(context))
                                .build();
                instanceManager.trackAd(nativeAd, call.<Integer>argument("adId"));
                nativeAd.load();
                result.success(null);
                break;
            case "loadAppLovinNativeAd":
                if (flutterAppLovinAds.getAppLovinSdk() != null) {
                    final String appLovinfactoryId = call.argument("factoryId");
                    final NativeAdFactory appLovinfactory = nativeAdFactories.get(appLovinfactoryId);
                    if (appLovinfactory == null) {
                        final String appLovinmessage = String.format("Can't find NativeAdFactory with id: %s", appLovinfactoryId);
                        result.error("NativeAdError", appLovinmessage, null);
                        break;
                    }

                    Boolean isAppLovinMediumNativeAds = call.<Boolean>argument("isMediumNativeAds");
                    isAppLovinMediumNativeAds = isAppLovinMediumNativeAds == null ? false :isAppLovinMediumNativeAds;
                    final FlutterAppLovinNativeAd appLovinnativeAd =
                            new FlutterAppLovinNativeAd.Builder()
                                    .setManager(instanceManager)
                                    .setAppLovinSdk(flutterAppLovinAds.getAppLovinSdk())
                                    .setAdUnitId(call.<String>argument("adUnitId"))
                                    .IsMediumNativeAds(isAppLovinMediumNativeAds)
                                    .setAdFactory(appLovinfactory)
                                    .setId(call.<Integer>argument("adId"))
                                    .setFlutterAdLoader(new FlutterAdLoader(context))
                                    .build();
                    instanceManager.trackAd(appLovinnativeAd, call.<Integer>argument("adId"));
                    appLovinnativeAd.load();
                    result.success(null);
                }  else {
                    instanceManager.onAdFailedToLoad(call.<Integer>argument("adId"), new FlutterAd.FlutterLoadAdError(new LoadAdError(999, "AppLovin SDK not Intitalized", "applovin.com", null, null)));
                    result.success(null);
//                    result.error("InvalidRequest", , null);
                }
                break;
            case "loadFacebookNativeAd":
                final String facebookfactoryId = call.argument("factoryId");
                final NativeAdFactory facebookfactory = nativeAdFactories.get(facebookfactoryId);
                if (facebookfactory == null) {
                    final String appLovinmessage = String.format("Can't find NativeAdFactory with id: %s", facebookfactoryId);
                    result.error("NativeAdError", appLovinmessage, null);
                    break;
                }

                Boolean isFacebookMediumNativeAds = call.<Boolean>argument("isMediumNativeAds");
                isFacebookMediumNativeAds = isFacebookMediumNativeAds == null ? false : isFacebookMediumNativeAds;
                final FlutterFacebookNativeAd facebooknativeAd =
                        new FlutterFacebookNativeAd.Builder()
                                .setManager(instanceManager)
                                .setAdUnitId(call.<String>argument("adUnitId"))
//                                .setAdUnitId("IMG_16_9_APP_INSTALL#YOUR_PLACEMENT_ID")
                                .IsMediumNativeAds(isFacebookMediumNativeAds)
                                .setAdFactory(facebookfactory)
                                .setId(call.<Integer>argument("adId"))
                                .setFlutterAdLoader(new FlutterAdLoader(context))
                                .build();
                instanceManager.trackAd(facebooknativeAd, call.<Integer>argument("adId"));
                facebooknativeAd.load();
                result.success(null);
                break;
            case "loadCustomNativeAd":
                Log.e(TAG, "onMethodCall:  ass daksd kas dkasdk ask d 11111111" );
                Boolean isCustomMediumNativeAds = call.<Boolean>argument("isMediumNativeAds");
                isCustomMediumNativeAds = isCustomMediumNativeAds == null ? false : isCustomMediumNativeAds;
                final FlutterCustomNativeAd customnativeAd =
                        new FlutterCustomNativeAd.Builder()
                                .setManager(instanceManager)
                                .setAdUnitId(call.<String>argument("adUnitId"))
//                                .setAdUnitId("IMG_16_9_APP_INSTALL#YOUR_PLACEMENT_ID")
                                .IsMediumNativeAds(isCustomMediumNativeAds)
                                .setId(call.<Integer>argument("adId"))
                                .setFlutterAdLoader(new FlutterAdLoader(context))
                                .build();
                instanceManager.trackAd(customnativeAd, call.<Integer>argument("adId"));
                customnativeAd.load();
                result.success(null);
                break;
            case "loadInterstitialAd":
                final FlutterInterstitialAd interstitial =
                        new FlutterInterstitialAd(
                                call.<Integer>argument("adId"),
                                instanceManager,
                                call.<String>argument("adUnitId"),
                                call.<FlutterAdRequest>argument("request"),
                                new FlutterAdLoader(context));
                instanceManager.trackAd(interstitial, call.<Integer>argument("adId"));
                interstitial.load();
                result.success(null);
                break;
            case "loadFacebookInterstitialAd":
                final FlutterFacebookInterstitialAd fbInterstitial =
                        new FlutterFacebookInterstitialAd(
                                call.<Integer>argument("adId"),
                                instanceManager,
                                call.<String>argument("adUnitId"),
//                                "IMG_16_9_APP_INSTALL#YOUR_PLACEMENT_ID",
                                new FlutterAdLoader(context));
                instanceManager.trackAd(fbInterstitial, call.<Integer>argument("adId"));
                fbInterstitial.load();
                result.success(null);
                break;
            case "loadCustomInterstitialAd":
                Log.e(TAG, "onMethodCall:    loadCustomInterstitialAd  -------- " );
                final FlutterCustomInterstitialAd customInterstitial =
                        new FlutterCustomInterstitialAd(
                                call.<Integer>argument("adId"),
                                instanceManager,
                                call.<String>argument("adUnitId"),
                                new FlutterAdLoader(context));
                instanceManager.trackAd(customInterstitial, call.<Integer>argument("adId"));
                customInterstitial.load();
                result.success(null);
                break;
            case "loadAppLovinInterstitialAd":
                if (flutterAppLovinAds.getAppLovinSdk() != null) {
                    final FlutterAppLovinInterstitialAd appLovininterstitial = new FlutterAppLovinInterstitialAd(
                            call.<Integer>argument("adId"),
                            instanceManager,
                            call.<String>argument("adUnitId"),
                            flutterAppLovinAds.getAppLovinSdk(),
                            new FlutterAdLoader(context));
                    instanceManager.trackAd(appLovininterstitial, call.<Integer>argument("adId"));
                    appLovininterstitial.load();
                    result.success(null);
                } else {
                    instanceManager.onAdFailedToLoad(call.<Integer>argument("adId"), new FlutterAd.FlutterLoadAdError(new LoadAdError(999, "AppLovin SDK not Intitalized", "applovin.com", null, null)));
                    result.success(null);
//                    result.error("InvalidRequest", , null);
                }
                break;
            case "loadRewardedAd":
                final String rewardedAdUnitId = requireNonNull(call.<String>argument("adUnitId"));
                final FlutterAdRequest rewardedAdRequest = call.argument("request");
                final FlutterAdManagerAdRequest rewardedAdManagerRequest =
                        call.argument("adManagerRequest");

                final FlutterRewardedAd rewardedAd;
                if (rewardedAdRequest != null) {
                    rewardedAd =
                            new FlutterRewardedAd(
                                    call.<Integer>argument("adId"),
                                    requireNonNull(instanceManager),
                                    rewardedAdUnitId,
                                    rewardedAdRequest,
                                    new FlutterAdLoader(context));
                } else if (rewardedAdManagerRequest != null) {
                    rewardedAd =
                            new FlutterRewardedAd(
                                    call.<Integer>argument("adId"),
                                    requireNonNull(instanceManager),
                                    rewardedAdUnitId,
                                    rewardedAdManagerRequest,
                                    new FlutterAdLoader(context));
                } else {
                    result.error("InvalidRequest", "A null or invalid ad request was provided.", null);
                    break;
                }

                instanceManager.trackAd(rewardedAd, requireNonNull(call.<Integer>argument("adId")));
                rewardedAd.load();
                result.success(null);
                break;
            case "loadFacebookRewardedAd":
                final String fbrewardedAdUnitId = /*"IMG_16_9_APP_INSTALL#YOUR_PLACEMENT_ID"; //*/requireNonNull(call.<String>argument("adUnitId"));

                final FlutterFacebookRewardedAd fbrewardedAd = new FlutterFacebookRewardedAd(
                        call.<Integer>argument("adId"),
                        requireNonNull(instanceManager),
                        fbrewardedAdUnitId,
                        new FlutterAdLoader(context));
                instanceManager.trackAd(fbrewardedAd, requireNonNull(call.<Integer>argument("adId")));
                fbrewardedAd.load();
                result.success(null);
                break;
            case "loadAppLovinRewardedAd":
                if (flutterAppLovinAds.getAppLovinSdk() != null) {
                    final String appLovinrewardedAdUnitId = requireNonNull(call.<String>argument("adUnitId"));

                    final FlutterAppLovinRewardedAd appLovinrewardedAd =
                            new FlutterAppLovinRewardedAd(
                                    call.<Integer>argument("adId"),
                                    requireNonNull(instanceManager),
                                    appLovinrewardedAdUnitId,
                                    new FlutterAdLoader(context),
                                    flutterAppLovinAds.getAppLovinSdk());

                    instanceManager.trackAd(appLovinrewardedAd, requireNonNull(call.<Integer>argument("adId")));
                    appLovinrewardedAd.load();
                    result.success(null);
                } else {
                    instanceManager.onAdFailedToLoad(call.<Integer>argument("adId"), new FlutterAd.FlutterLoadAdError(new LoadAdError(999, "AppLovin SDK not Intitalized", "applovin.com", null, null)));
                    result.success(null);
                }
                break;
            case "loadAdManagerBannerAd":
                final FlutterAdManagerBannerAd adManagerBannerAd =
                        new FlutterAdManagerBannerAd(
                                call.<Integer>argument("adId"),
                                instanceManager,
                                call.<String>argument("adUnitId"),
                                call.<List<FlutterAdSize>>argument("sizes"),
                                call.<FlutterAdManagerAdRequest>argument("request"),
                                getBannerAdCreator(context));
                instanceManager.trackAd(adManagerBannerAd, call.<Integer>argument("adId"));
                adManagerBannerAd.load();
                result.success(null);
                break;
            case "loadFluidAd":
                final FluidAdManagerBannerAd fluidAd =
                        new FluidAdManagerBannerAd(
                                call.<Integer>argument("adId"),
                                instanceManager,
                                call.<String>argument("adUnitId"),
                                call.<FlutterAdManagerAdRequest>argument("request"),
                                getBannerAdCreator(context));
                instanceManager.trackAd(fluidAd, call.<Integer>argument("adId"));
                fluidAd.load();
                result.success(null);
                break;
            case "loadAdManagerInterstitialAd":
                final FlutterAdManagerInterstitialAd adManagerInterstitialAd =
                        new FlutterAdManagerInterstitialAd(
                                call.<Integer>argument("adId"),
                                requireNonNull(instanceManager),
                                requireNonNull(call.<String>argument("adUnitId")),
                                call.<FlutterAdManagerAdRequest>argument("request"),
                                new FlutterAdLoader(context));
                instanceManager.trackAd(
                        adManagerInterstitialAd, requireNonNull(call.<Integer>argument("adId")));
                adManagerInterstitialAd.load();
                result.success(null);
                break;
            case "loadRewardedInterstitialAd":
                final String rewardedInterstitialAdUnitId =
                        requireNonNull(call.<String>argument("adUnitId"));
                final FlutterAdRequest rewardedInterstitialAdRequest = call.argument("request");
                final FlutterAdManagerAdRequest rewardedInterstitialAdManagerRequest =
                        call.argument("adManagerRequest");

                final FlutterRewardedInterstitialAd rewardedInterstitialAd;
                if (rewardedInterstitialAdRequest != null) {
                    rewardedInterstitialAd =
                            new FlutterRewardedInterstitialAd(
                                    call.<Integer>argument("adId"),
                                    requireNonNull(instanceManager),
                                    rewardedInterstitialAdUnitId,
                                    rewardedInterstitialAdRequest,
                                    new FlutterAdLoader(context));
                } else if (rewardedInterstitialAdManagerRequest != null) {
                    rewardedInterstitialAd =
                            new FlutterRewardedInterstitialAd(
                                    call.<Integer>argument("adId"),
                                    requireNonNull(instanceManager),
                                    rewardedInterstitialAdUnitId,
                                    rewardedInterstitialAdManagerRequest,
                                    new FlutterAdLoader(context));
                } else {
                    result.error("InvalidRequest", "A null or invalid ad request was provided.", null);
                    break;
                }

                instanceManager.trackAd(
                        rewardedInterstitialAd, requireNonNull(call.<Integer>argument("adId")));
                rewardedInterstitialAd.load();
                result.success(null);
                break;
            case "loadAppOpenAd":
                final FlutterAppOpenAd appOpenAd =
                        new FlutterAppOpenAd(
                                call.<Integer>argument("adId"),
                                call.<Integer>argument("orientation"),
                                requireNonNull(instanceManager),
                                requireNonNull(call.<String>argument("adUnitId")),
                                call.<FlutterAdRequest>argument("request"),
                                call.<FlutterAdManagerAdRequest>argument("adManagerRequest"),
                                new FlutterAdLoader(context));
                instanceManager.trackAd(appOpenAd, call.<Integer>argument("adId"));
                appOpenAd.load();
                result.success(null);
                break;
            case "loadAppLovinOpenAd":
                if (flutterAppLovinAds.getAppLovinSdk() != null) {
                    final FlutterAppLovinOpenAd appLovinOpenAd =
                            new FlutterAppLovinOpenAd(
                                    call.<Integer>argument("adId"),
                                    requireNonNull(instanceManager),
                                    requireNonNull(call.<String>argument("adUnitId")),
                                    flutterAppLovinAds.getAppLovinSdk(),
                                    new FlutterAdLoader(context));
                    instanceManager.trackAd(appLovinOpenAd, call.<Integer>argument("adId"));
                    appLovinOpenAd.load();
                    result.success(null);
                } else {
                    instanceManager.onAdFailedToLoad(call.<Integer>argument("adId"), new FlutterAd.FlutterLoadAdError(new LoadAdError(999, "AppLovin SDK not Intitalized", "applovin.com", null, null)));
                    result.success(null);
//                    result.error("InvalidRequest", , null);
                }
                break;
            case "disposeAd":
                instanceManager.disposeAd(call.<Integer>argument("adId"));
                result.success(null);
                break;
            case "showAdWithoutView":
                final boolean adShown = instanceManager.showAdWithId(call.<Integer>argument("adId"));
                if (!adShown) {
                    result.error("AdShowError", "Ad failed to show.", null);
                    break;
                }
                result.success(null);
                break;
            case "AdSize#getAnchoredAdaptiveBannerAdSize":
                final FlutterAdSize.AnchoredAdaptiveBannerAdSize size =
                        new FlutterAdSize.AnchoredAdaptiveBannerAdSize(
                                context,
                                new FlutterAdSize.AdSizeFactory(),
                                call.<String>argument("orientation"),
                                call.<Integer>argument("width"));
                if (AdSize.INVALID.equals(size.size)) {
                    result.success(null);
                } else {
                    result.success(size.height);
                }
                break;
            case "MobileAds#setAppMuted":
                flutterMobileAds.setAppMuted(call.<Boolean>argument("muted"));
                result.success(null);
                break;
            case "AppLovinAds#setAppMuted":
                flutterAppLovinAds.setAppMuted(call.<Boolean>argument("muted"));
                result.success(null);
                break;
            case "MobileAds#setAppVolume":
                flutterMobileAds.setAppVolume(call.<Double>argument("volume"));
                result.success(null);
                break;
            case "setImmersiveMode":
                ((FlutterOverlayAd) instanceManager.adForId(call.<Integer>argument("adId")))
                        .setImmersiveMode(call.<Boolean>argument("immersiveModeEnabled"));
                result.success(null);
                break;
            case "MobileAds#disableMediationInitialization":
                flutterMobileAds.disableMediationInitialization(context);
                result.success(null);
                break;
            case "MobileAds#getVersionString":
                result.success(flutterMobileAds.getVersionString());
                break;
            case "MobileAds#openDebugMenu":
                String adUnitId = call.argument("adUnitId");
                flutterMobileAds.openDebugMenu(context, adUnitId);
                result.success(null);
                break;
            case "getAdSize": {
                FlutterAd ad = instanceManager.adForId(call.<Integer>argument("adId"));
                if (ad == null) {
                    // This was called on a dart ad container that hasn't been loaded yet.
                    result.success(null);
                } else if (ad instanceof FlutterBannerAd) {
                    result.success(((FlutterBannerAd) ad).getAdSize());
                } else if (ad instanceof FlutterAdManagerBannerAd) {
                    result.success(((FlutterAdManagerBannerAd) ad).getAdSize());
                }  else if (ad instanceof FlutterAppLovinBannerAd) {
                    result.success(((FlutterAppLovinBannerAd) ad).getAdSize());
                } else {
                    result.error(
                            Constants.ERROR_CODE_UNEXPECTED_AD_TYPE,
                            "Unexpected ad type for getAdSize: " + ad,
                            null);
                }
                break;
            }
            case "setServerSideVerificationOptions": {
                FlutterAd ad = instanceManager.adForId(call.<Integer>argument("adId"));
                final FlutterServerSideVerificationOptions options =
                        call.argument("serverSideVerificationOptions");
                if (ad == null) {
                    Log.w(TAG, "Error - null ad in setServerSideVerificationOptions");
                } else if (ad instanceof FlutterRewardedAd) {
                    ((FlutterRewardedAd) ad).setServerSideVerificationOptions(options);
                } else if (ad instanceof FlutterRewardedInterstitialAd) {
                    ((FlutterRewardedInterstitialAd) ad).setServerSideVerificationOptions(options);
                } else {
                    Log.w(TAG, "Error - setServerSideVerificationOptions called on " + "non-rewarded ad");
                }
                result.success(null);
                break;
            }
            default:
                result.notImplemented();
        }
    }

    @VisibleForTesting
    BannerAdCreator getBannerAdCreator(@NonNull Context context) {
        return new BannerAdCreator(context);
    }

    /**
     * An {@link OnInitializationCompleteListener} that invokes result.success() at most once.
     */
    private static final class FlutterInitializationListener
            implements OnInitializationCompleteListener {

        private final Result result;
        private boolean isInitializationCompleted;

        private FlutterInitializationListener(@NonNull final Result result) {
            this.result = result;
            isInitializationCompleted = false;
        }

        @Override
        public void onInitializationComplete(@NonNull InitializationStatus initializationStatus) {
            // Make sure not to invoke this more than once, since Dart will throw an exception if success
            // is invoked more than once. See b/193418432.
            if (isInitializationCompleted) {
                return;
            }
            result.success(new FlutterInitializationStatus(initializationStatus));
            isInitializationCompleted = true;
        }
    }

    private static final class FlutterAppLovinInitializationListener
            implements AppLovinSdk.SdkInitializationListener {

        private final Result result;
        private boolean isInitializationCompleted;

        private FlutterAppLovinInitializationListener(@NonNull final Result result) {
            this.result = result;
            isInitializationCompleted = false;
        }

        @Override
        public void onSdkInitialized(AppLovinSdkConfiguration config) {
            // Make sure not to invoke this more than once, since Dart will throw an exception if success
            // is invoked more than once. See b/193418432.
            if (isInitializationCompleted) {
                return;
            }
            result.success(new FlutterAppLovinInitializationStatus(config));
            isInitializationCompleted = true;
        }
    }
}
