// Copyright 2022 Bitcoding


package com.btcpiyush.ads.google_applovin_unity_ads;

import android.os.Bundle;

import androidx.annotation.Nullable;

import com.google.android.gms.ads.mediation.MediationExtrasReceiver;

import java.util.Map;

import io.flutter.embedding.engine.FlutterEngine;

/**
 * Provides network specific parameters when constructing an ad request.
 *
 * <p>After you register a {@code MediationNetworkExtrasProvider} using {@link
 * GoogleApplovinUnityAdsPlugin#registerMediationNetworkExtrasProvider(FlutterEngine,
 * MediationNetworkExtrasProvider)}, the plugin will use it to pass extras when constructing ad
 * requests.
 */
public interface MediationNetworkExtrasProvider {

  /**
   * Gets mediation extras that should be included for an ad request with the {@code adUnitId} and
   * {@code identifier}, as a map from adapter classes to their corresponding extras bundle. You can
   * refer to each network's <a
   * href="https://developers.google.com/admob/android/mediation/applovin#network-specific_parameters">documentation</a>
   * to see how to find the corresponding adapter class and create its extras. This will be called
   * whenever the plugin creates an ad request.
   *
   * @param adUnitId The ad unit for the associated ad request.
   * @param identifier An optional identifier that comes from the associated dart ad request object.
   *     This allows for additional control of which extras to include for an ad request, beyond
   *     just the ad unit.
   * @return A map of mediation adapter classes to their extras that should be included in the ad
   *     request. If {@code null} is returned then no extras are added.
   */
  @Nullable
  Map<Class<? extends MediationExtrasReceiver>, Bundle> getMediationExtras(
      String adUnitId, @Nullable String identifier);
}
