// Copyright 2022 Bitcoding
// See the License for the specific language governing permissions and
// limitations under the License.


package com.btcpiyush.ads.google_applovin_unity_ads.usermessagingplatform;

import android.content.Context;

import androidx.annotation.Nullable;

import com.google.android.ump.ConsentRequestParameters;

import java.util.Objects;

/** Wrapper for {@link ConsentRequestParameters}. */
class ConsentRequestParametersWrapper {

  @Nullable private final Boolean tfuac;

  @Nullable private final ConsentDebugSettingsWrapper debugSettings;

  ConsentRequestParametersWrapper(
      @Nullable Boolean tfuac, @Nullable ConsentDebugSettingsWrapper debugSettings) {
    this.tfuac = tfuac;
    this.debugSettings = debugSettings;
  }

  @Nullable
  Boolean getTfuac() {
    return tfuac;
  }

  @Nullable
  ConsentDebugSettingsWrapper getDebugSettings() {
    return debugSettings;
  }

  /** Get as {@link ConsentRequestParameters}. */
  ConsentRequestParameters getAsConsentRequestParameters(Context context) {
    ConsentRequestParameters.Builder builder = new ConsentRequestParameters.Builder();
    if (tfuac != null) {
      builder.setTagForUnderAgeOfConsent(tfuac);
    }
    if (debugSettings != null) {
      builder.setConsentDebugSettings(debugSettings.getAsConsentDebugSettings(context));
    }
    return builder.build();
  }

  @Override
  public boolean equals(@Nullable Object obj) {
    if (obj == this) {
      return true;
    } else if (!(obj instanceof ConsentRequestParametersWrapper)) {
      return false;
    }

    ConsentRequestParametersWrapper other = (ConsentRequestParametersWrapper) obj;
    return Objects.equals(tfuac, other.getTfuac())
        && Objects.equals(debugSettings, other.getDebugSettings());
  }

  @Override
  public int hashCode() {
    return Objects.hash(tfuac, debugSettings);
  }
}
