// Copyright 2022 Bitcoding

import 'package:flutter/material.dart';
import 'package:google_applovin_unity_ads/google_applovin_unity_ads.dart';
import 'package:google_applovin_unity_ads_example/Model.dart';
import 'package:visibility_detector/visibility_detector.dart';

class MainScreen extends StatefulWidget {
  const MainScreen({Key? key}) : super(key: key);

  @override
  _MyAppState createState() => _MyAppState();
}

class _MyAppState extends State<MainScreen> {
  Widget _bannerAd = Container(height: 0);
  final nativeAdDistance = 7;

  final String placeholderText = """
      Hello This example screen is second screen.
      """;

  @override
  void initState() {
    super.initState();
    loadAdsinList();
    showBannerAds(
        size: AdSize.leaderboard,
        onAdLoadedCallback: (bannerAds) => {
              setState(() {
                _bannerAd = bannerAds;
              })
            });
  }

  @override
  void dispose() {
    super.dispose();
    destroyAd();
  }

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      home: Builder(builder: (BuildContext context) {
        return Scaffold(
          appBar: AppBar(
            title: const Text('Ads example app'),
            actions: <Widget>[
              PopupMenuButton<String>(
                onSelected: (String result) {
                  switch (result) {
                    case 'InterstitialAd':
                      showIntraAds(
                          callback: () => {print("Show Completed true")});
                      break;
                    case 'RewardedAd':
                      showRewardAds(
                          callback: () => {print("Show Completed true")});
                      break;
                    default:
                      throw AssertionError('unexpected button: $result');
                  }
                },
                itemBuilder: (BuildContext context) => <PopupMenuEntry<String>>[
                  const PopupMenuItem<String>(
                    value: 'InterstitialAd',
                    child: Text('InterstitialAd'),
                  ),
                  const PopupMenuItem<String>(
                    value: 'RewardedAd',
                    child: Text('RewardedAd'),
                  )
                ],
              ),
            ],
          ),
          body: SafeArea(
              child: showAdsInListView(
                  bannerSize: AdSize.leaderboard,
                  controller: AdsController.instance,
                  adsDistance: nativeAdDistance,
                  widgetMargin: EdgeInsets.symmetric(horizontal: 8, vertical: 8),
                  itemBuilder: (context, index) {
                    Widget widget = Container(height: 0);
                    if (AdsController.instance.widgetlist[index] is Model) {
                      widget = ListTile(
                        title: Text(
                            (AdsController.instance.widgetlist[index] as Model)
                                .label),
                      );
                    }
                    return VisibilityDetector(
                        key: Key(index.toString()),
                        onVisibilityChanged: (VisibilityInfo info) {
                          if (info.visibleFraction == 1) {
                            AdsController.instance.currentIndex.value = index;
                          }
                        },
                        child: widget
                    );
                  },
                  isNativeAds: false)),
          bottomNavigationBar: _bannerAd,
        );
      }),
    );
  }

  loadAdsinList() {
    for (var i = 0; i < 500; i++) {
      AdsController.instance.widgetlist.add(Model("Default Test \nad sdhakjsh kali\t das"));
    }
  }
}