import 'dart:convert';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:google_applovin_unity_ads/ads/SettingsModel.dart';
import 'package:google_applovin_unity_ads/google_applovin_unity_ads.dart';
import 'package:http/http.dart' as http;
import 'package:shared_preferences/shared_preferences.dart';

Settings? _settings;

InterstitialAd? _interstitialAd;
AppLovinInterstitialAd? _appLovininterstitialAd;
FacebookInterstitialAd? _facebookInterstitialAd;
CustomInterstitialAd? _customInterstitialAd;
AdManagerInterstitialAd? _adManagerInterstitialAd;

AppLifecycleReactor? _appLifecycleReactor;

RewardedAd? _rewardedAd;
Widget? _progressDialog;
AppLovinRewardedAd? _appLovinrewardedAd;
FacebookRewardedAd? _facebookrewardedAd;
int _appVersionCode = 1;
bool _isRewardShowCompleted = false;

int _interestialAdTimeInterval = 0;

Future<Settings?> fetchAdsSettings(
    {required int appVersionCode,
    required String settingsUrl,
    String? keyName}) async {
  final response = await http.get(Uri.parse(settingsUrl));
  if (response.statusCode == 200) {
    return convertAdsSettings(
        appVersionCode: appVersionCode,
        jsonData: response.body,
        keyName: keyName);
  } else {
    return null;
  }
}

Future<Settings?> convertAdsSettings(
    {required int appVersionCode,
    required String jsonData,
    String? keyName}) async {
  _appVersionCode = appVersionCode;
  if (keyName == null || keyName == "") {
    _settings = Settings.fromJson(jsonDecode(jsonData));
  } else {
    _settings = _Response.fromJson(jsonDecode(jsonData), keyName).response;
  }
  return _settings;
}

int getAppVersionCode() {
  return _appVersionCode;
}

Settings? getAdsSettings() {
  return _settings;
}

InterstitialAd? getInterestitialAd() {
  return _interstitialAd;
}

AppLovinInterstitialAd? getAppLovinInterestitialAd() {
  return _appLovininterstitialAd;
}

FacebookInterstitialAd? getFacebookInterestitialAd() {
  return _facebookInterstitialAd;
}

CustomInterstitialAd? getCustomInterestitialAd() {
  return _customInterstitialAd;
}

AdManagerInterstitialAd? getAdManagerInterstitialAd() {
  return _adManagerInterstitialAd;
}

AppLifecycleReactor? getAppLifecycleReactor() {
  return _appLifecycleReactor;
}

RewardedAd? getRewardedAd() {
  return _rewardedAd;
}

AppLovinRewardedAd? getAppLovinRewardedAd() {
  return _appLovinrewardedAd;
}

FacebookRewardedAd? getFacebookRewardedAd() {
  return _facebookrewardedAd;
}

int getInterestialAdTimeInterval() {
  return _interestialAdTimeInterval;
}

Widget getProgressDialogWidget() {
  return _progressDialog ?? WillPopScope(
    onWillPop: () async {
      return false;
    },
    child: Dialog(
      child: Container(
        padding:
        const EdgeInsets.symmetric(vertical: 18.0, horizontal: 12.0),
        child: Row(
          mainAxisSize: MainAxisSize.min,
          children: [
            const CircularProgressIndicator(),
            Container(
                padding: const EdgeInsets.only(left: 18.0),
                child: Text(
                  _settings?.adSetting?.adsLoadingText ?? "Ads Loading",
                  style: const TextStyle(fontSize: 20),
                )),
          ],
        ),
      ),
    ),
  );
}

void setProgressWidget(Widget? wdt) {
  _progressDialog = wdt;
}

bool IsRewardShowCompleted() {
  return _isRewardShowCompleted;
}

void setInterestitialAd(InterstitialAd? _intraAd) {
  if (_intraAd != null) {
    setTimerDelayed();
  } else {
    resetTimerDelayed();
  }
  _interstitialAd = _intraAd;
}

void incrementCounter() {
  _settings?.adSetting?.interestialAdsIntervalCount =
      (_settings?.adSetting?.interestialAdsIntervalCount ?? 0) + 1;
}

incrementIntraSequence() async {
  // Obtain shared preferences.
  final prefs = await SharedPreferences.getInstance();
  final curSeq  = prefs.getInt('intraAdsCurSeqPos') ?? 0;
  await prefs.setInt('intraAdsCurSeqPos', curSeq + 1);
}

Future<int> currentIntraSequence() async {
  // Obtain shared preferences.
  final prefs = await SharedPreferences.getInstance();
  final int? counter = prefs.getInt('intraAdsCurSeqPos');
  return counter ?? 0;
}

incrementRewardSequence() async {
  // Obtain shared preferences.
  final prefs = await SharedPreferences.getInstance();
  final curSeq  = prefs.getInt('rewardAdsCurSeqPos') ?? 0;
  await prefs.setInt('rewardAdsCurSeqPos', curSeq + 1);
}

Future<int> currentRewardSequence() async {
  // Obtain shared preferences.
  final prefs = await SharedPreferences.getInstance();
  final int? counter = prefs.getInt('rewardAdsCurSeqPos');
  return counter ?? 0;
}

Future<void> incrementBannerSequence() async {
  // Obtain shared preferences.
  final prefs = await SharedPreferences.getInstance();
  final curSeq  = prefs.getInt('bannerAdsCurSeqPos') ?? -1;
  await prefs.setInt('bannerAdsCurSeqPos', curSeq + 1);
  // _settings?.adSetting?.bannerAdsStep = (_settings?.adSetting?.bannerAdsStep ?? 0) + 1;
}

Future<int> currentBannerSequence() async {
  final prefs = await SharedPreferences.getInstance();
  final curSeq  = prefs.getInt('bannerAdsCurSeqPos') ?? -1;
  return curSeq;
  // return (_settings?.adSetting?.bannerAdsStep ?? 0);
}

Future<void> incrementNativeSequence() async {
  final prefs = await SharedPreferences.getInstance();
  final curSeq  = prefs.getInt('nativeAdsCurSeqPos') ?? -1;
  await prefs.setInt('nativeAdsCurSeqPos', curSeq + 1);
  // _settings?.adSetting?.nativeAdsStep = (_settings?.adSetting?.nativeAdsStep ?? -1) + 1;
}

Future<int> currentNativeSequence() async {
  final prefs = await SharedPreferences.getInstance();
  final curSeq  = prefs.getInt('nativeAdsCurSeqPos') ?? -1;
  return curSeq;

  // return (_settings?.adSetting?.nativeAdsStep ?? 0);
}

incrementIntraAdsCustomImpressionCount(bool isCmpressionTrue) {
  if (isCmpressionTrue) {
    _settings?.adSetting?.interestialAdsCustomImpressionCount = (_settings?.adSetting?.interestialAdsCustomImpressionCount ?? 0) + 1;
  }
}

int currentIntraAdsCustomImpression() {
  return (_settings?.adSetting?.interestialAdsCustomImpressionCount ?? 0);
}

void setFacebookInterestitialAd(FacebookInterstitialAd? _intraAd) {
  if (_intraAd != null) {
    setTimerDelayed();
  } else {
    resetTimerDelayed();
  }
  _facebookInterstitialAd = _intraAd;
}

void setCustomInterestitialAd(CustomInterstitialAd? _intraAd) {
  if (_intraAd != null) {
    setTimerDelayed();
  } else {
    resetTimerDelayed();
  }
  _customInterstitialAd = _intraAd;
}

void setAppLovinInterestitialAd(AppLovinInterstitialAd? _intraAd) {
  if (_intraAd != null) {
    setTimerDelayed();
  } else {
    resetTimerDelayed();
  }
  _appLovininterstitialAd = _intraAd;
}

void setAdManagerInterstitialAd(AdManagerInterstitialAd? admanagerIntraAd) {
  if (admanagerIntraAd != null) {
    setTimerDelayed();
  } else {
    resetTimerDelayed();
  }
  _adManagerInterstitialAd = admanagerIntraAd;
}

void setAppLifecycleReactor(AppLifecycleReactor? lifecycle) {
  _appLifecycleReactor = lifecycle;
}

void setRewardedAd(RewardedAd? reward) {
  if (reward != null) {
    setTimerDelayed();
  } else {
    resetTimerDelayed();
  }
  _rewardedAd = reward;
}

void setAppLovinRewardedAd(AppLovinRewardedAd? reward) {
  if (reward != null) {
    setTimerDelayed();
  } else {
    resetTimerDelayed();
  }
  _appLovinrewardedAd = reward;
}

void setFacebookRewardedAd(FacebookRewardedAd? reward) {
  if (reward != null) {
    setTimerDelayed();
  } else {
    resetTimerDelayed();
  }
  _facebookrewardedAd = reward;
}

void setRewardShowCompleted(bool isrewardComplted) {
  _isRewardShowCompleted = isrewardComplted;
}

void setInterestialAdTimeInterval(int timeInterval) {
  _interestialAdTimeInterval = timeInterval;
}

void showProgressbarDialog(BuildContext context) {
  showDialog(
      context: context,
      barrierDismissible: false,
      builder: (BuildContext context) {
        return getProgressDialogWidget();
      });
}

class _Response {
  Settings? response;

  _Response({this.response});

  _Response.fromJson(Map<String, dynamic> json, String keyName) {
    response = json[keyName] != null ? Settings.fromJson(json[keyName]) : null;
  }
}
