// Copyright 2022 Bitcoding
// See the License for the specific language governing permissions and
// limitations under the License.

import 'package:google_applovin_unity_ads/google_applovin_unity_ads.dart';

import 'ad_inspector_containers.dart';
import 'ad_instance_manager.dart';
import 'request_configuration.dart';
import 'package:flutter/foundation.dart';

/// Class contains logic that applies to the Google Mobile Ads SDK as a whole.
///
/// Right now, the only methods in it are used for initialization.
///
/// See [instance].
class AppLovinAds {
  AppLovinAds._();

  static final AppLovinAds _instance = AppLovinAds._().._init();

  /// Shared instance to initialize the AdMob SDK.
  static AppLovinAds get instance => _instance;

  /// Initializes the AppLovin Ads SDK.
  ///
  /// Call this method as early as possible after the app launches to reduce
  /// latency on the session's first ad request.
  ///
  /// If this method is not called, the first ad request automatically
  /// initializes the AppLovin Ads SDK.
  Future<AppLovinInitializationStatus> initialize(String sdkKey, String  geoedgeSdkKey, int  geoedgeSdkKeyTimeoutInSeconds) {
    return instanceManager.applovinInitialize(sdkKey, geoedgeSdkKey, geoedgeSdkKeyTimeoutInSeconds);
  }

  /// Get the current [RequestConfiguration].
  ///
  /// On iOS, tagForUnderAgeOfConsent and tagForChildDirectedTreatment are null on
  /// the returned [RequestConfiguration].
  Future<RequestConfiguration> getRequestConfiguration() {
    return instanceManager.getRequestConfiguration();
  }


  /// Sets whether the app is muted.
  ///
  /// For more details about the volume control, visit
  /// https://developers.google.com/admob/android/global-settings#video_ad_volume_control
  Future<void> setAppMuted(bool muted) {
    return instanceManager.setAppLovinAppMuted(muted);
  }

  /// Internal init to cleanup state for hot restart.
  /// This is a workaround for https://github.com/flutter/flutter/issues/7160.
  void _init() {
    instanceManager.channel.invokeMethod('_init');
  }
}

/// The status of the SDK initialization.
class AppLovinInitializationStatus {
  /// Default constructor to create an [InitializationStatus].
  ///
  /// Returned when calling [MobileAds.initialize];
  AppLovinInitializationStatus(Map<String, String> adapterStatuses)
      : adapterStatuses = Map<String, String>.unmodifiable(
    adapterStatuses,
  );

  /// Initialization status of each known ad network, keyed by its adapter's class name.
  final Map<String, String> adapterStatuses;
}